FUNCTION  FS (T)

  USE PRECISION, ONLY:                                                 &
                       WP
  USE CONTROL_DATA, ONLY:                                              &
                       DT
  USE AUXIL    , ONLY:                                                 &
                       PI
  USE SOURCE   , ONLY:                                                 &
                       NSIG,                                           &
                       TP, TS, FREQ,                                   &
                       A1, A2, A3, NT, STF

  IMPLICIT NONE

  INTEGER :: I

  REAL (WP)             :: A, B, C, FS
  REAL (WP), INTENT(IN) :: T


  IF ( (T.LT.0.) .OR. (T.GT.(NT-1)*DT) ) THEN
    FS = 0.
  END IF

  IF     (NSIG.EQ.1) THEN
!____________________________________________________________ KUPPER ___

    A=T*A1
    B=T*A2
    FS=SIN(A)-.5*SIN(B)
  ELSE IF (NSIG.EQ.2) THEN
!____________________________________________________________ RICKER ___

    A=T-TS
    A=(6.*A*A)/A1
    FS=A2*(A-.5)*EXP(-A)
  ELSE IF (NSIG.EQ.3) THEN
!_____________________________________________________________ GABOR ___

    A=T-TS
    FS=EXP(-A2*A*A)*COS(A1*A+A3)
  ELSE IF (NSIG.EQ.4) THEN
!___________________________________________________________ BERLAGE ___

    FS=(T**A3)*EXP(-A2*T)*SIN(A1*T)
  ELSE IF (NSIG.EQ.5) THEN
!_________________________________________________ ZERO-PHASE RICKER ___
    A=T-TS
    FS=EXP(-A1*A*A)*COS(A2*A)
  ELSE IF (NSIG.EQ.6) THEN
!_______________________________ DERIVATIVE OF THE ZERO-PHASE RICKER ___

    A=T-TP
    B=A1*A*A
    C=A2*A
    FS=-1.*EXP(B)*FREQ*( FREQ*A*COS(C) + PI*SIN(C) )

  ELSE IF (NSIG.EQ.0) THEN
 !___________________________________ SOURCE_TIME_FUNCTION_FROM_FILE ___

    I = INT(T/DT) + 1
    FS = STF(I)
  ENDIF

END FUNCTION  FS