!=======================================================================
! I=2 and MX1
!    XXXT   (I,0   ) =  xi^xx (I,0)
!    TXXT   (I)      = tau_xx (I,0)
! I=2 and MX1
!    XZZT   (I,0   ) =  xi^zz (I,0)
!    XZZT   (I,1   ) =  xi^zz (I,1)
!    TZZMWT (I,LZZ1) = tau_zz (I,0)
!    TZZMWT (I,LZZ2) = tau_zz (I,1)
SUBROUTINE  FS0_S_2ND  (  JMT, MX1T,                                   &
                          MUT,        LAMT ,                           &
                         YMUT,       YLAMT ,                           &
                           UT,   WT,                                   &
                         XXXT,       XZZT  ,                           &
                         TXXT,       TZZMWT  )

  USE PRECISION   , ONLY:                                              &
                          WP,IK
  USE CONTROL_DATA, ONLY:                                              &
                          KEY_1D
  USE AUXIL       , ONLY:                                              &
                          OTA
  USE T_INTERFACES, ONLY:                                              &
                          IND

  IMPLICIT NONE

  INTEGER     ,                  INTENT(IN)    ::                      &
                                                  MX1T
  INTEGER (IK), DIMENSION(:,0:), INTENT(IN)    ::                      &
                                                  JMT
  REAL    (WP), DIMENSION(:)   , INTENT(IN)    ::                      &
                                                   MUT,  LAMT
  REAL    (WP), DIMENSION(:,:) , INTENT(IN)    ::                      &
                                                  YMUT, YLAMT
  REAL    (WP), DIMENSION(:)   , INTENT(OUT)   ::                      &
                                                  TXXT
  REAL    (WP), DIMENSION(:,:) , INTENT(OUT)   ::                      &
                                                  TZZMWT
  REAL    (WP), DIMENSION(:,0:), INTENT(IN)    ::                      &
                                                  UT, WT
  REAL    (WP), DIMENSION(:,0:), INTENT(INOUT) ::                      &
                                                  XXXT, XZZT

  INTEGER :: I, L, LP1


  L   = 0

  LP1 = L + 1

  I = 2

  IF ( .NOT. KEY_1D ) THEN
    CALL  FS0_TXXT_2ND
  END IF
  CALL  FS0_TZZT_2ND

  I = MX1T

  IF ( .NOT. KEY_1D ) THEN
    CALL  FS0_TXXT_2ND
  END IF
  CALL  FS0_TZZT_2ND


CONTAINS

!-----------------------------------------------------------------------
  SUBROUTINE  FS0_TXXT_2ND

    INTEGER      :: JM1, IO
    REAL    (WP) :: LAMIL, LA2MI, YLAMIL, YLA2MI, XT, XT0, OT, ET1, ET2

! coarse graining
    IO     = IND (I,L)
    OT     = OTA (IO)
    JM1    = JMT (I,L)
    YLAMIL = YLAMT (JM1,IO)
     LAMIL =  LAMT (JM1)
    YLA2MI = YLAMIL + 2._WP*YMUT (JM1,IO)
     LA2MI =  LAMIL + 2._WP* MUT (JM1)
! u_x (I,0) == u_x (I+1/2,1/2)
    ET1    = UT (I+1,L  ) - UT (I  ,L  )
! w_z (I,0) == w_z (I+1/2,1/2)
    ET2    = WT (I  ,LP1) - WT (I  ,L  )
! xi^xx (I,0) == xi^xx (I+1/2,1/2)
    XT0    = XXXT (I,L)
    XT     = OT*XT0 + YLA2MI * ET1 + YLAMIL * ET2
    XXXT (I,L) = XT
! tau_xx (I) == tau_xx (I+1/2,1/2)
    TXXT (I)   = LA2MI * ET1 +  LAMIL * ET2 - ( XT + XT0 ) * 0.5_WP

  END SUBROUTINE  FS0_TXXT_2ND

!-----------------------------------------------------------------------
  SUBROUTINE  FS0_TZZT_2ND

    INTEGER      :: JM1, IO, LZZ1, LZZ2, LL, LLP1
    REAL    (WP) :: LAMIL, LA2MI, YLAMIL, YLA2MI, XT, XT0, OT,         &
                    ET1, ET2

    LZZ1 = 1

! coarse graining
    IO     = IND (I,L)
    OT     = OTA (IO)
    JM1    = JMT (I,L)
    YLAMIL = YLAMT (JM1,IO)
     LAMIL =  LAMT (JM1)
    YLA2MI = YLAMIL + 2._WP*YMUT (JM1,IO)
     LA2MI =  LAMIL + 2._WP* MUT (JM1)
! u_x (I,0) == u_x (I+1/2,1/2)
    ET1    = UT (I+1,L  ) - UT (I  ,L  )
! w_z (I,0) == w_z (I+1/2,1/2)
    ET2    = WT (I  ,LP1) - WT (I  ,L  )
! xi^zz (I,0) == xi^zz (I+1/2,1/2)
    XT0    = XZZT (I,L)
    XT     = OT*XT0 + YLAMIL * ET1 + YLA2MI * ET2
    XZZT   (I,L   ) = XT
! tau_zz (I,LZZ1) = tau_zz (I+1/2,1/2)
    TZZMWT (I,LZZ1) = LAMIL * ET1 + LA2MI * ET2                        &
                    - ( XT + XT0 ) * 0.5_WP
    IF ( KEY_1D ) THEN
      IO     = IND (I+1,L)
      OT     = OTA (IO)
      JM1    = JMT (I,L)
      YLAMIL = YLAMT (JM1,IO)
       LAMIL =  LAMT (JM1)
      YLA2MI = YLAMIL + 2._WP*YMUT (JM1,IO)
       LA2MI =  LAMIL + 2._WP* MUT (JM1)
      XT0    = XXXT (I,L)
      XT     = OT*XT0 + YLAMIL * ET1 + YLA2MI * ET2
      XXXT   (I,L   ) = XT

      TZZMWT (I,LZZ1) = TZZMWT (I,LZZ1)                                &
                      - ( XT + XT0 ) * 0.5_WP
    END IF


    LZZ2 = 2

    LL   = L  + 1
    LLP1 = LL + 1

! coarse graining
    IO     = IND (I,LL)
    OT     = OTA (IO)
    JM1    = JMT (I,LL)
    YLAMIL = YLAMT (JM1,IO)
     LAMIL =  LAMT (JM1)
    YLA2MI = YLAMIL + 2._WP*YMUT (JM1,IO)
     LA2MI =  LAMIL + 2._WP* MUT (JM1)
! u_x (I,1) == u_x (I+1/2,3/2)
    ET1    = UT (I+1,LL  ) - UT (I  ,LL  )
! w_z (I,1) == w_z (I+1/2,3/2)
    ET2    = WT (I  ,LLP1) - WT (I  ,LL  )
! xi^zz (I,1) == xi^zz (I+1/2,3/2)
    XT0    = XZZT (I,LL)
    XT     = OT*XT0 + YLAMIL * ET1 + YLA2MI * ET2
    XZZT   (I,LL  ) = XT
! tau_zz (I,LZZ2) = tau_zz (I+1/2,3/2)
    TZZMWT (I,LZZ2) = LAMIL * ET1 + LA2MI * ET2                        &
                    - ( XT + XT0 ) * 0.5_WP
    IF ( KEY_1D ) THEN
      IO     = IND (I+1,L)
      OT     = OTA (IO)
      JM1    = JMT (I,L)
      YLAMIL = YLAMT (JM1,IO)
       LAMIL =  LAMT (JM1)
      YLA2MI = YLAMIL + 2._WP*YMUT (JM1,IO)
       LA2MI =  LAMIL + 2._WP* MUT (JM1)
      XT0    = XXXT (I,L)
      XT     = OT*XT0 + YLAMIL * ET1 + YLA2MI * ET2
      XXXT   (I,L   ) = XT

      TZZMWT (I,LZZ2) = TZZMWT (I,LZZ2)                                &
                      - ( XT + XT0 ) * 0.5_WP
    END IF

  END SUBROUTINE  FS0_TZZT_2ND

END SUBROUTINE  FS0_S_2ND
