!=======================================================================
! I=3,...,MX2
!    XXXT   (I,L   ) =  xi^xx (I,L  )
!    TXXT   (I)      = tau_xx (I,L  )
! I=3,...,MX2
!    XZZT   (I,L+1 ) =  xi^zz (I,L+1)
!    TZZMWT (I,LZZ4) = tau_zz (I,L+1)
! I=3,...,MX1
!    XZXT   (I,L+2 ) =  xi^xz (I,L+2)
!    TZXMWT (I,LZX4) = tau_xz (I,L+2)
SUBROUTINE  HOM_S_INT  (    L,   MX1T,   MX2T,                         &
                           UT,     WT,                                 &
                         XXXT, XZXT  , XZZT  ,                         &
                         TXXT, TZXMWT, TZZMWT  )

  USE PRECISION   , ONLY:                                              &
                          WP
  USE CONTROL_DATA, ONLY:                                              &
                          KEY_1D
  USE GRID_MEDIUM , ONLY:                                              &
                          COMU, COLAM, COL2M, CYMU, CYLAM, CYL2M
  USE AUXIL       , ONLY:                                              &
                          OTA, A, B
  USE T_INTERFACES, ONLY:                                              &
                          IND

  IMPLICIT NONE

  INTEGER     ,                  INTENT(IN)    ::                      &
                                                  L, MX1T, MX2T
  REAL    (WP), DIMENSION(:)   , INTENT(OUT)   ::                      &
                                                  TXXT
  REAL    (WP), DIMENSION(:,:) , INTENT(OUT)   ::                      &
                                                  TZXMWT, TZZMWT
  REAL    (WP), DIMENSION(:,0:), INTENT(IN)    ::                      &
                                                  UT, WT
  REAL    (WP), DIMENSION(:,0:), INTENT(INOUT) ::                      &
                                                  XXXT, XZXT, XZZT

  INTEGER      :: I, IO,        LM1,      LP1,  LP2,                   &
                         LLM2, LLM1, LL, LLP1, LLP2,  LZX4, LZZ4
  REAL    (WP) :: OT, ET1, ET2, XT, XT0


  LM1 = L - 1
  LP1 = L + 1
  LP2 = L + 2

  IF ( .NOT. KEY_1D ) THEN
    DO  I = 3, MX2T

! coarse graining
      IO  = IND (I,L)
      OT  = OTA (IO)
! u_x (I,L) == u_x (I+1/2,L+1/2)
      ET1 = ( A*( UT (I+2,L  ) - UT (I-1,L  ) )                        &
            + B*( UT (I+1,L  ) - UT (I  ,L  ) ) )
! w_z (I,L) == w_z (I+1/2,L+1/2)
      ET2 = ( A*( WT (I  ,LP2) - WT (I  ,LM1) )                        &
            + B*( WT (I  ,LP1) - WT (I  ,L  ) ) )
! xi^xx (I,L) == xi^xx (I+1/2,L+1/2)
      XT0 = XXXT (I,L)
      XT  = OT*XT0 + CYL2M (IO) * ET1 + CYLAM (IO) * ET2
      XXXT (I,L) = XT
! tau_xx (I) == tau_xx (I+1/2,L+1/2)
      TXXT (I)   = COL2M * ET1 + COLAM * ET2 - ( XT + XT0 )*0.5_WP

    END DO
  END IF

!_______________________________________________________________________

  LZZ4 = MOD( (MOD(L+1,4) + 4), 4 ) + 1

  LL   = L  + 1

  LLM1 = LL - 1
  LLP1 = LL + 1
  LLP2 = LL + 2

  DO  I = 3, MX2T

! coarse graining
    IO  = IND (I, LL)
    OT  = OTA (IO)
! u_x (I,LL) == u_x (I+1/2,L+3/2)
    ET1 = ( A*( UT (I+2,LL  ) - UT (I-1,LL  ) )                        &
          + B*( UT (I+1,LL  ) - UT (I  ,LL  ) ) )
! w_z (I,LL) == w_z (I+1/2,L+3/2)
    ET2 = ( A*( WT (I  ,LLP2) - WT (I  ,LLM1) )                        &
          + B*( WT (I  ,LLP1) - WT (I  ,LL  ) ) )
! xi^zz (I,LL) == xi^zz (I+1/2,L+3/2)
    XT0 = XZZT (I,LL)
    XT  = OT*XT0 + CYLAM (IO) * ET1 + CYL2M (IO) * ET2
    XZZT   (I,LL  ) = XT
! tau_zz (I,LZZ4) == tau_zz (I+1/2,L+3/2)
    TZZMWT (I,LZZ4) = COLAM * ET1 + COL2M * ET2                        &
                    - ( XT + XT0 )*0.5_WP

    IF ( KEY_1D ) THEN
      IO     = IND (I+1,LL)
      OT     = OTA (IO)
      XT0    = XXXT (I,LL)
      XT     = OT*XT0 + CYLAM (IO) * ET1 + CYL2M (IO) * ET2
      XXXT   (I,LL  ) = XT

      TZZMWT (I,LZZ4) = TZZMWT (I,LZZ4)                                &
                      - ( XT + XT0 ) * 0.5_WP
    END IF

  END DO

!_______________________________________________________________________

  IF ( .NOT. KEY_1D ) THEN

    LZX4 = MOD( (MOD(L+2,4) + 3), 4 ) + 1

    LL   = L  + 2

    LLM2 = LL - 2
    LLM1 = LL - 1
    LLP1 = LL + 1

    DO  I = 3, MX1T

! coarse graining
      IO  = IND (I,LL)
      OT  = OTA (IO)
! u_z (I,LL) + w_x (I,LL) == u_z (I,L+2) + w_x (I,L+2)
      ET1 = ( A*( UT (I  ,LLP1) - UT (I  ,LLM2)                        &
                + WT (I+1,LL  ) - WT (I-2,LL  ) )                      &
            + B*( UT (I  ,LL  ) - UT (I  ,LLM1)                        &
                + WT (I  ,LL  ) - WT (I-1,LL  ) ) )
! xi^xz (I,LL) == xi^xz (I,L+2)
      XT0 = XZXT (I,LL)
      XT  = OT*XT0 + CYMU (IO) * ET1
      XZXT   (I,LL  ) = XT
! tau_xz (I,LZX4) == tau_xz (I,L+2)
      TZXMWT (I,LZX4) = COMU * ET1 - ( XT + XT0 )*0.5_WP

    END DO

  END IF

END SUBROUTINE  HOM_S_INT
