!=======================================================================
! I=2 and MX1
!    XXXT   (I,L   ) =  xi^xx (I,L  )
!    TXXT   (I)      = tau_xx (I,L  )
! I=2 and MX1
!    XZZT   (I,L   ) =  xi^zz (I,L  )
!    TZZMWT (I,LZZ3) = tau_zz (I,L  )
! I=2 and MX
!    XZXT   (I,L   ) =  xi^xz (I,L  )
!    TZXMWT (I,LZX2) = tau_xz (I,L  )
SUBROUTINE  INH_S_2ND  (    L,    JMT,    MXT, MX1T,                   &
                          MUT,  MUXZT,   LAMT,                         &
                         YMUT, YMUXZT,  YLAMT,                         &
                           UT,     WT,                                 &
                         XXXT, XZXT  , XZZT  ,                         &
                         TXXT, TZXMWT, TZZMWT        )

  USE PRECISION   , ONLY:                                              &
                          WP, IK
  USE CONTROL_DATA, ONLY:                                              &
                          KEY_1D
  USE AUXIL       , ONLY:                                              &
                          OTA
  USE T_INTERFACES, ONLY:                                              &
                          IND

  IMPLICIT NONE

  INTEGER     ,                  INTENT(IN)    ::                      &
                                                  L, MXT, MX1T
  INTEGER (IK), DIMENSION(:,0:), INTENT(IN)    ::                      &
                                                  JMT
  REAL    (WP), DIMENSION(:)   , INTENT(IN)    ::                      &
                                                   MUT,  LAMT,  MUXZT
  REAL    (WP), DIMENSION(:,:) , INTENT(IN)    ::                      &
                                                  YMUT, YLAMT, YMUXZT
  REAL    (WP), DIMENSION(:)   , INTENT(OUT)   ::                      &
                                                  TXXT
  REAL    (WP), DIMENSION(:,:) , INTENT(OUT)   ::                      &
                                                  TZXMWT, TZZMWT
  REAL    (WP), DIMENSION(:,0:), INTENT(IN)    ::                      &
                                                  UT, WT
  REAL    (WP), DIMENSION(:,0:), INTENT(INOUT) ::                      &
                                                  XXXT, XZXT, XZZT

  INTEGER :: I, LM1, LP1

  LM1 = L - 1
  LP1 = L + 1

  I = 2

  IF ( .NOT. KEY_1D ) THEN
    CALL  INH_TXXT_2ND
    CALL  INH_TZXT_2ND
  END IF
  CALL  INH_TZZT_2ND

  I = MX1T

  CALL  INH_TZZT_2ND
  IF ( .NOT. KEY_1D ) THEN
    CALL  INH_TXXT_2ND

    I = MXT

    CALL  INH_TZXT_2ND
  END IF

CONTAINS

!-----------------------------------------------------------------------
  SUBROUTINE  INH_TXXT_2ND

    INTEGER      :: JM1, IO
    REAL    (WP) :: LAMIL, LA2MI, YLAMIL, YLA2MI, OT, ET1, ET2, XT, XT0

! coarse graining
    IO     = IND (I,L)
    OT     = OTA (IO)
    JM1    = JMT (I,L)
    YLAMIL = YLAMT (JM1,IO)
     LAMIL =  LAMT (JM1)
    YLA2MI = YLAMIL + 2._WP*YMUT (JM1,IO)
     LA2MI =  LAMIL + 2._WP* MUT (JM1)
! u_x (I,L) == u_x (I+1/2,L+1/2)
    ET1    = UT (I+1,L  ) - UT (I  ,L  )
! w_z (I,L) == w_z (I+1/2,L+1/2)
    ET2    = WT (I  ,LP1) - WT (I  ,L  )
! xi^xx (I,L) == xi^xx (I+1/2,L+1/2)
    XT0    = XXXT (I,L)
    XT     = OT*XT0 + YLA2MI * ET1 + YLAMIL * ET2
    XXXT (I,L) = XT
! tau_xx (I) == tau_xx (I+1/2,L+1/2)
    TXXT (I)   = LA2MI * ET1 + LAMIL * ET2 - ( XT + XT0 )*0.5_WP

  END SUBROUTINE  INH_TXXT_2ND

!-----------------------------------------------------------------------
  SUBROUTINE  INH_TZZT_2ND

    INTEGER      :: JM1, IO, LZZ3
    REAL    (WP) :: LAMIL, LA2MI, YLAMIL, YLA2MI, OT, ET1, ET2, XT, XT0

    LZZ3 = MOD( (MOD(L+1,4) + 3), 4 ) + 1

! coarse graining
    IO     = IND (I,L)
    OT     = OTA (IO)
    JM1    = JMT (I,L)
    YLAMIL = YLAMT (JM1,IO)
     LAMIL =  LAMT (JM1)
    YLA2MI = YLAMIL + 2._WP*YMUT (JM1,IO)
     LA2MI =  LAMIL + 2._WP* MUT (JM1)
! u_x (I,L) == u_x (I+1/2,L+1/2)
    ET1    = UT (I+1,L  ) - UT (I  ,L  )
! w_z (I,L) == w_z (I+1/2,L+1/2)
    ET2    = WT (I  ,LP1) - WT (I  ,L  )
! xi^zz (I,L) == xi^zz (I+1/2,L+1/2)
    XT0    = XZZT (I,L)
    XT     = OT*XT0 + YLAMIL * ET1 + YLA2MI * ET2
    XZZT   (I,L   ) = XT
! tau_zz (I,LZZ3) == tau_zz (I+1/2,L+1/2)
    TZZMWT (I,LZZ3) = LAMIL * ET1 + LA2MI * ET2                        &
                    - ( XT + XT0 )*0.5_WP

    IF ( KEY_1D ) THEN
      IO     = IND (I+1,L)
      OT     = OTA (IO)
      JM1    = JMT (I,L)
      YLAMIL = YLAMT (JM1,IO)
       LAMIL =  LAMT (JM1)
      YLA2MI = YLAMIL + 2._WP*YMUT (JM1,IO)
       LA2MI =  LAMIL + 2._WP* MUT (JM1)
      XT0    = XXXT (I,L)
      XT     = OT*XT0 + YLAMIL * ET1 + YLA2MI * ET2
      XXXT   (I,L   ) = XT

      TZZMWT (I,LZZ3) = TZZMWT (I,LZZ3)                                &
                      - ( XT + XT0 ) * 0.5_WP
    END IF

  END SUBROUTINE  INH_TZZT_2ND

!-----------------------------------------------------------------------
  SUBROUTINE  INH_TZXT_2ND

    INTEGER      :: JM1, IO, LZX2
    REAL    (WP) :: OT, ET1, XT, XT0

    LZX2 = MOD( (MOD(L+2,4) + 1), 4 ) + 1

! coarse graining
    IO  = IND (I,L)
    OT  = OTA (IO)
    JM1 = JMT (I,L)
! u_z (I,L) + w_x (I,L) == u_z (I,L) + w_x (I,L)
    ET1 = UT (I  ,L  ) - UT (I  ,LM1)                                  &
        + WT (I  ,L  ) - WT (I-1,L  )
! xi^xz (I,L) == xi^xz (I,L)
    XT0 = XZXT (I,L)
    XT  = OT*XT0 + YMUXZT (JM1,IO) * ET1
    XZXT   (I,L   ) = XT
! tau_xz (I,LZX2) == tau_xz (I,L)
    TZXMWT (I,LZX2) = MUXZT (JM1) * ET1 - ( XT + XT0 )*0.5_WP

  END SUBROUTINE  INH_TZXT_2ND

END SUBROUTINE  INH_S_2ND
