!=======================================================================
! I=2 and MX1
!    XZZT   (I,L   ) =  xi^zz (I,MZ1)
!    TZZMWT (I,LZZ3) = tau_zz (I,MZ1)
! I=2 and MX
!    XZXT   (I,L   ) =  xi^xz (I,MZ1)
!    TZXMWT (I,LZX2) = tau_xz (I,MZ1)
SUBROUTINE  MZ1_S_2ND  ( MXT,   MX1T,                                  &
                          UT,     WT,                                  &
                              XZXT  , XZZT  ,                          &
                              TZXMWT, TZZMWT  )

  USE PRECISION   , ONLY:                                              &
                          WP
  USE CONTROL_DATA, ONLY:                                              &
                          KEY_1D
  USE GRID_MEDIUM , ONLY:                                              &
                          COMU, COLAM, COL2M, CYMU, CYLAM, CYL2M
  USE WAVEFIELD   , ONLY:                                              &
                          XXX
  USE AUXIL       , ONLY:                                              &
                          OTA, MZ1
  USE T_INTERFACES, ONLY:                                              &
                          IND

  IMPLICIT NONE

  INTEGER     ,                  INTENT(IN)    ::                      &
                                                  MXT, MX1T
  REAL    (WP), DIMENSION(:,:) , INTENT(OUT)   ::                      &
                                                  TZXMWT, TZZMWT
  REAL    (WP), DIMENSION(:,0:), INTENT(IN)    ::                      &
                                                  UT, WT
  REAL    (WP), DIMENSION(:,0:), INTENT(INOUT) ::                      &
                                                  XZXT, XZZT

  INTEGER :: L, I, LM1, LP1


  L   = MZ1

  LM1 = L - 1
  LP1 = L + 1

  I = 2

  CALL  HOM_TZZT_2ND
  IF ( .NOT. KEY_1D ) THEN
    CALL  HOM_TZXT_2ND
  END IF

  I = MX1T

  CALL  HOM_TZZT_2ND

  I = MXT

  IF ( .NOT. KEY_1D ) THEN
    CALL  HOM_TZXT_2ND
  END IF

CONTAINS

!-----------------------------------------------------------------------
  SUBROUTINE  HOM_TZZT_2ND

    INTEGER      :: IO, LZZ3
    REAL    (WP) :: OT, ET1, ET2, XT, XT0

    LZZ3 = MOD( (MOD(L+1,4) + 3), 4 ) + 1

! coarse graining
    IO  = IND (I,L)
    OT  = OTA (IO)
! u_x (I,L) == u_x (I+1/2,L+1/2)
    ET1 = UT (I+1,L  ) - UT (I  ,L  )
! w_z (I,L) == w_z (I+1/2,L+1/2)
    ET2 = WT (I  ,LP1) - WT (I  ,L  )
! xi^zz (I,L) == xi^zz (I+1/2,L+1/2)
    XT0 = XZZT (I,L)
    XT  = OT*XT0 + CYLAM (IO) * ET1 + CYL2M (IO) * ET2
    XZZT   (I,L   ) = XT
! tau_zz (I,LZZ3) == tau_zz (I+1/2,L+1/2)
    TZZMWT (I,LZZ3) = COLAM * ET1 + COL2M * ET2                        &
                    - ( XT + XT0 )*0.5_WP

    IF ( KEY_1D ) THEN
      IO     = IND (I+1,L)
      OT     = OTA (IO)
      XT0    = XXX (I,L)
      XT     = OT*XT0 + CYLAM (IO) * ET1 + CYL2M (IO) * ET2
      XXX (I,L  ) = XT

      TZZMWT (I,LZZ3) = TZZMWT (I,LZZ3)                                &
                      - ( XT + XT0 ) * 0.5_WP
    END IF

    END SUBROUTINE  HOM_TZZT_2ND

!-----------------------------------------------------------------------
  SUBROUTINE  HOM_TZXT_2ND

    INTEGER      :: IO, LZX2
    REAL    (WP) :: OT, ET1, XT, XT0

    LZX2 = MOD( (MOD(L+2,4) + 1), 4 ) + 1

! coarse graining
    IO  = IND (I,L)
    OT  = OTA (IO)
! u_z (I,L) + w_x (I,L) == u_z (I,L) + w_x (I,L)
    ET1 = UT (I  ,L  ) - UT (I  ,LM1)                                  &
        + WT (I  ,L  ) - WT (I-1,L  )
! xi^xz (I,L) == xi^xz (I,L)
    XT0 = XZXT (I,L)
    XT  = OT*XT0 + CYMU (IO) * ET1
    XZXT   (I,L   ) = XT
! tau_xz (I,LZX2) == tau_xz (I,L)
    TZXMWT (I,LZX2) = COMU * ET1 - ( XT + XT0 )*0.5_WP

  END SUBROUTINE  HOM_TZXT_2ND

END SUBROUTINE  MZ1_S_2ND
