!-----------------------------------------------------------------------

SUBROUTINE  NONREF ( H, KEY, ALFA, BETA, THPP, THSS,                   &
                     AX01, AX02, AX10, AX11, AX12, AX20, AX21, AX22 )

  USE PRECISION   , ONLY:                                              &
                          WP, DP

  USE CONTROL_DATA, ONLY:                                              &
                          DT, OMG, WB

  IMPLICIT NONE

  INTEGER     , INTENT (IN)  :: KEY

  REAL    (WP), INTENT (IN)  :: H, ALFA, BETA, THPP, THSS

  REAL    (WP), INTENT (OUT) :: AX01, AX02, AX10, AX11,                &
                                AX12, AX20, AX21, AX22

  REAL    (DP) :: GPLUS, GMINUS, DTAUX, HAUX, AA00, AA01, AA02,        &
                                              AA10, AA11, AA12,        &
                                              AA20, AA21, AA22
  REAL    (WP) :: TMP, TMP1, TMP2

!----------------------------------------------------------------------

  INTERFACE

    SUBROUTINE  HIGDON ( WB, ALFA, BETA, DT, H,                        &
                         A00, A01, A02, A10, A11, A12, A20, A21, A22 )
      USE PRECISION, ONLY: WP, DP
      IMPLICIT NONE
      REAL (WP), INTENT (IN)  :: WB, ALFA, BETA, DT, H
      REAL (DP), INTENT (OUT) :: A00, A01, A02,                        &
                                 A10, A11, A12,                        &
                                 A20, A21, A22
    END SUBROUTINE  HIGDON


    SUBROUTINE  OptAbsBc ( GPLUS, GMINUS, DT, DX,                      &
                           A00, A01, A02, A10, A11, A12, A20, A21, A22 )

      REAL*8 GPLUS, GMINUS, DT, DX
      REAL*8 A00, A01, A02, A10, A11, A12, A20, A21, A22
    END SUBROUTINE  OptAbsBc

  END INTERFACE

!----------------------------------------------------------------------

    SELECT CASE ( KEY )

!____________________________________________________ Solid Boundary ___
    CASE ( 0 )
      AX01 =  0._WP
      AX02 =  0._WP
      AX10 =  0._WP
      AX11 =  0._WP
      AX12 =  0._WP
      AX20 =  0._WP
      AX21 =  0._WP
      AX22 =  0._WP

!____________________________________________________________ Higdon ___
    CASE ( 1 )
      CALL HIGDON ( WB, ALFA, BETA, DT, H,                             &
                    AA00,AA01,AA02,AA10,AA11,AA12,AA20,AA21,AA22 )
      AX01 = REAL ( AA01, WP )
      AX02 = REAL ( AA02, WP )
      AX10 = REAL ( AA10, WP )
      AX11 = REAL ( AA11, WP )
      AX12 = REAL ( AA12, WP )
      AX20 = REAL ( AA20, WP )
      AX21 = REAL ( AA21, WP )
      AX22 = REAL ( AA22, WP )

!__________________________________________ Reynolds, max. P-w. att. ___
    CASE ( 2 )
      TMP   = -DT*ALFA/H
      AX01 =  0._WP
      AX02 =  0._WP
      AX10 =  1._WP + TMP
      AX11 =  1._WP - TMP
      AX12 =  0._WP
      AX20 =  0._WP
      AX21 = -1._WP - TMP
      AX22 =          TMP

!__________________________________________ Reynolds, max. S-w. att. ___
    CASE ( 3 )
      TMP   = -DT*BETA/H
      AX01 =  0._WP
      AX02 =  0._WP
      AX10 =  1._WP + TMP
      AX11 =  1._WP - TMP
      AX12 =  0._WP
      AX20 =  0._WP
      AX21 = -1._WP - TMP
      AX22 =          TMP

!_____________________________________ Peng & Toksoz, max. P-w. att. ___
    CASE ( 4 )
      GPLUS = DBLE ( THPP       )
      GMINUS= DBLE ( THSS       )
      DTAUX = DBLE ( OMG*DT     )
       HAUX = DBLE ( OMG/ALFA*H )
      CALL OptAbsBc ( GPLUS, GMINUS, DTAUX, HAUX,                      &
                      AA00,AA01,AA02,AA10,AA11,AA12,AA20,AA21,AA22 )
      AX01 = REAL ( AA01 )
      AX02 = REAL ( AA02 )
      AX10 = REAL ( AA10 )
      AX11 = REAL ( AA11 )
      AX12 = REAL ( AA12 )
      AX20 = REAL ( AA20 )
      AX21 = REAL ( AA21 )
      AX22 = REAL ( AA22 )

!_____________________________________ Peng & Toksoz, max. S-w. att. ___
    CASE ( 5 )
      GPLUS = DBLE ( THPP       )
      GMINUS= DBLE ( THSS       )
      DTAUX = DBLE ( OMG*DT     )
       HAUX = DBLE ( OMG/BETA*H )
      CALL OptAbsBc ( GPLUS, GMINUS, DTAUX, HAUX,                      &
                      AA00,AA01,AA02,AA10,AA11,AA12,AA20,AA21,AA22 )
      AX01 = REAL ( AA01 )
      AX02 = REAL ( AA02 )
      AX10 = REAL ( AA10 )
      AX11 = REAL ( AA11 )
      AX12 = REAL ( AA12 )
      AX20 = REAL ( AA20 )
      AX21 = REAL ( AA21 )
      AX22 = REAL ( AA22 )

!_________________________________ Emerman & Stephen, max. P-w. att. ___
    CASE ( 6 )
      TMP1 = ( DT - H/ALFA )/( DT + H/ALFA )
      TMP2 =  2._WP*H/ALFA  /( DT + H/ALFA )
      AX01 =  TMP1
      AX02 =  0._WP
      AX10 =  TMP2
      AX11 =  TMP2
      AX12 =  0._WP
      AX20 =  TMP1
      AX21 = -1._WP
      AX22 =  0._WP

!_________________________________ Emerman & Stephen, max. S-w. att. ___
    CASE ( 7 )
      TMP1 = ( DT - H/BETA )/( DT + H/BETA )
      TMP2 =  2._WP*H/BETA  /( DT + H/BETA )
      AX01 =  TMP1
      AX02 =  0._WP
      AX10 =  TMP2
      AX11 =  TMP2
      AX12 =  0._WP
      AX20 =  TMP1
      AX21 = -1._WP
      AX22 =  0._WP

    END SELECT

END SUBROUTINE NONREF
