!=======================================================================

SUBROUTINE  PRE_SOURCE

  USE PRECISION   , ONLY :                                             &
                           WP
  USE CONTROL_DATA, ONLY:                                              &
                          DT, COVP, COVS, KEY_SH, KEY_1D
  USE GRID_MEDIUM , ONLY:                                              &
                          H
  USE SOURCE      , ONLY:                                              &
                          NSIG, INC, NT, ISR, LSR,                     &
                          TP, TS, A1, A2, A3, FREQ,                    &
                          GAMA, PSI, ZETA, ETA, EPSILON,               &
                          M0, DIP, MXX, MZX, MZZ, STF_FILE, STF
  USE AUXIL       , ONLY:                                              &
                          PI

  IMPLICIT NONE

  INTEGER :: I, OFFS

  READ (16,*) NSIG, INC

  IF ( KEY_1D ) THEN
    INC = 0
  END IF
  IF ( KEY_SH ) THEN
    INC = 0
  END IF

  IF (  INC .EQ. 2 ) READ (16,*) ISR, LSR
  IF (  INC .EQ. 3 ) READ (16,*) ISR, LSR
  IF (  INC .EQ. 4 )   THEN
                     READ (16,*) ISR, LSR
                     READ (16,*) M0, DIP
  END IF
  IF (  INC .EQ. 5 )   THEN
                     READ (16,*) ISR, LSR
                     READ (16,*) M0
  END IF

  IF ( NSIG .EQ. 0 ) READ (16,*) STF_FILE, NT
  IF ( NSIG .EQ. 1 ) READ (16,*)   TP
  IF ( NSIG .EQ. 2 ) READ (16,*)   TP,                      TS
  IF ( NSIG .EQ. 3 ) READ (16,*) GAMA,          FREQ,  PSI, TS
  IF ( NSIG .EQ. 4 ) READ (16,*) GAMA,          FREQ, ZETA, TS
  IF ( NSIG .EQ. 5 ) READ (16,*)  ETA, EPSILON, FREQ,       TS
  IF ( NSIG .EQ. 6 ) READ (16,*)   TP,          FREQ,       TS

  IF ( NSIG .EQ. 1 ) THEN
 !___________________________________________________________ KUPPER ___

    A1 = 2. * PI/TP
    A2 = 2. * A1
    NT = TP/DT + 1
  ELSE IF ( NSIG .EQ. 2 ) THEN
 !___________________________________________________________ RICKER ___

    A1 = TP * SQRT(6.)/PI
    A1 = A1 * A1
    A2 = SQRT(PI)/2.
    IF( TS .LT. .0001 ) TS = 1.1 * TP
  ELSE IF ( NSIG .EQ. 3 ) THEN
 !____________________________________________________________ GABOR ___

    A1 = 2. * PI * FREQ
    A2 = A1/GAMA
    A2 = A2 * A2
    A3 = PSI
    IF ( TS .LT. .0001 ) TS = .45 * GAMA/FREQ
  ELSE IF ( NSIG .EQ. 4 ) THEN
 !__________________________________________________________ BERLAGE ___

    A1 = 2. * PI * FREQ
    A2 = A1/GAMA
    A3 = ZETA
    IF ( TS .LT. .0001 .AND. ZETA .EQ. 1. ) TS = 1.6 * GAMA/FREQ
  ELSE IF ( NSIG .EQ. 5 ) THEN
 !________________________________________________ ZERO-PHASE RICKER ___

    A1 = ETA * FREQ * FREQ
    A2 = EPSILON * PI * FREQ
  ELSE IF ( NSIG .EQ. 6 ) THEN
 !______________________________ DERIVATIVE OF THE ZERO-PHASE RICKER ___

    A1 = -1. * FREQ * FREQ/2.
    A2 =  PI * FREQ
    IF( TS .LT. .0001 ) TS = 1.5 * TP

  ELSE IF ( NSIG .EQ. 0 ) THEN
 !___________________________________ SOURCE_TIME_FUNCTION_FROM_FILE ___

    OPEN (74, FILE=STF_FILE, STATUS='OLD')
!      NT = 0
!      DO WHILE ( .NOT. EOF(74) )
!        READ (74,*)
!        NT = NT + 1
!      END DO
!      REWIND(74)
      IF      ( INC .EQ. 0 ) THEN
        OFFS = INT ( 5._WP * H/COVP/DT )
      ELSE IF ( INC .EQ. 1 ) THEN
        OFFS = INT ( 5._WP * H/COVS/DT )
      END IF

      ALLOCATE ( STF(-OFFS+1:NT+OFFS+1) )
      DO I = 1, NT
        READ (74,*) STF(I)
      END DO
      DO I = -OFFS+1, 0
        STF(I) = STF(1)
      END DO
      DO I = NT+1, NT+OFFS+1
        STF(I) = STF(NT)
      END DO
    CLOSE (74)

  ENDIF
 !______________________________________________________________________

  IF ( (NSIG .NE. 1) .AND. (NSIG .NE. 0) ) THEN
    NT = TS/DT + .5
    TS = DT * NT
    NT = NT + 1
    IF ( NSIG .NE. 4 ) THEN
      NT = 2 * NT - 1
    ENDIF
  ENDIF

!______________________________________________ DOUBLE-COUPLE SOURCE ___

  IF (INC .EQ. 4) THEN
    DIP = DIP * PI/180.
    MXX = - M0 * SIN(2*DIP) / (  H*H*H)
    MZX = - M0 * COS(2*DIP) / (4*H*H*H)
    MZZ =   M0 * SIN(2*DIP) / (  H*H*H)
  END IF

!__________________________________________________ EXPLOSIVE SOURCE ___

  IF (INC .EQ. 5) THEN
!    MXX = - M0 / (  H*H*H)
    MXX =   M0 / (  H*H*H) / 2.
    MZZ =   M0 / (  H*H*H) / 2.
  END IF

END SUBROUTINE  PRE_SOURCE
