!=======================================================================
! I=2 and MX1
!    XZZT   (I,LOU1) =  xi^zz (I,LOU1)
!    TZZMWT (I,LZZ2) = tau_zz (I,LOU1)
SUBROUTINE  PW_S_2ND  (          MX1T,                                 &
                           UT,     WT,                                 &
                                       XZZT  ,                         &
                                       TZZMWT  )

  USE PRECISION   , ONLY:                                              &
                          WP
  USE CONTROL_DATA, ONLY:                                              &
                          KEY_1D
  USE GRID_MEDIUM , ONLY:                                              &
                          COLAM, COL2M, CYLAM, CYL2M
  USE WAVEFIELD   , ONLY:                                              &
                          XXX
  USE AUXIL       , ONLY:                                              &
                          OTA, LOU1
  USE T_INTERFACES, ONLY:                                              &
                          IND

  IMPLICIT NONE

  INTEGER     ,                  INTENT(IN)    ::                      &
                                                  MX1T
  REAL    (WP), DIMENSION(:,:) , INTENT(OUT)   ::                      &
                                                  TZZMWT
  REAL    (WP), DIMENSION(:,0:), INTENT(IN)    ::                      &
                                                  UT, WT
  REAL    (WP), DIMENSION(:,0:), INTENT(INOUT) ::                      &
                                                  XZZT

  INTEGER      :: I, J, L, LP1


  L = LOU1

  LP1 = L + 1

  I = 2
  CALL  PW_TZZT_2ND

  I = MX1T
  CALL  PW_TZZT_2ND


CONTAINS

!-----------------------------------------------------------------------
  SUBROUTINE  PW_TZZT_2ND

    INTEGER      :: IO, LZZ2
    REAL    (WP) :: OT, ET1, ET2, XT, XT0

    LZZ2 = MOD( (MOD(L+2,4) + 2), 4 ) + 1

! coarse graining
    IO  = IND (I,L)
    OT  = OTA (IO)
! u_x (I,L) == u_x (I+1/2,L+1/2)
    ET1 = UT (I+1,L  ) - UT (I  ,L  )
! w_z (I,L) == w_z (I+1/2,L+1/2)
    ET2 = WT (I  ,LP1) - WT (I  ,L  )
! xi^zz (I,L) == xi^zz (I+1/2,L+1/2)
    XT0 = XZZT (I,L)
    XT  = OT*XT0 + CYLAM (IO) * ET1 + CYL2M (IO) * ET2
    XZZT   (I,L ) = XT
! tau_zz (I,LZZ2) == tau_zz (I+1/2,L+1/2)
    TZZMWT (I,LZZ2) = COLAM * ET1 + COL2M * ET2                        &
                    - ( XT + XT0 )*0.5_WP

    IF ( KEY_1D ) THEN
      IO     = IND (I+1,L)
      OT     = OTA (IO)
      XT0    = XXX (I,L)
      XT     = OT*XT0 + CYLAM (IO) * ET1 + CYL2M (IO) * ET2
      XXX   (I,L  ) = XT

      TZZMWT (I,LZZ2) = TZZMWT (I,LZZ2)                                &
                      - ( XT + XT0 ) * 0.5_WP
    END IF

  END SUBROUTINE  PW_TZZT_2ND

END SUBROUTINE  PW_S_2ND
