!=======================================================================
! I=3,...,MX2
!    XZZT   (I,LOU2) =  xi^zz (I,LOU2)
!    XZZT   (I,LOU1) =  xi^zz (I,LOU1)
!    XZZT   (I,LIN1) =  xi^zz (I,LIN1)
!    TZZMWT (I,LZZ1) = tau_zz (I,LOU2)
!    TZZMWT (I,LZZ2) = tau_zz (I,LOU1)
!    TZZMWT (I,LZZ3) = tau_zz (I,LIN1)
! I=3,...,MX1
!    XZXT   (I,LOU1) =  xi^xz (I,LOU1)
!    XZXT   (I,LIN1) =  xi^xz (I,LIN1)
!    XZXT   (I,LIN2) =  xi^xz (I,LIN2)
!    TZXMWT (I,LZX1) = tau_xz (I,LOU1)
!    TZXMWT (I,LZX2) = tau_xz (I,LIN1)
!    TZXMWT (I,LZX3) = tau_xz (I,LIN2)
SUBROUTINE  PW_S_INT  (        MX1T,   MX2T,                           &
                           UT,     WT,                                 &
                               XZXT  , XZZT  ,                         &
                               TZXMWT, TZZMWT  )

  USE PRECISION   , ONLY:                                              &
                          WP
  USE CONTROL_DATA, ONLY:                                              &
                          KEY_1D
  USE GRID_MEDIUM , ONLY:                                              &
                          COMU, COLAM, COL2M, CYMU, CYLAM, CYL2M
  USE WAVEFIELD   , ONLY:                                              &
                          XXX
  USE AUXIL       , ONLY:                                              &
                          OTA, A, B, LOU2, LOU1
  USE T_INTERFACES, ONLY:                                              &
                          IND

  IMPLICIT NONE

  INTEGER     ,                  INTENT(IN)    ::                      &
                                                  MX1T, MX2T
  REAL    (WP), DIMENSION(:,:) , INTENT(OUT)   ::                      &
                                                  TZXMWT, TZZMWT
  REAL    (WP), DIMENSION(:,0:), INTENT(IN)    ::                      &
                                                  UT, WT
  REAL    (WP), DIMENSION(:,0:), INTENT(INOUT) ::                      &
                                                  XZXT, XZZT

  INTEGER      :: I, J, IO, L, LLM2, LLM1, LL, LLP1, LLP2, LZZ, LZX
  REAL    (WP) :: OT, ET1, ET2, XT, XT0


  L = LOU2

  DO J = 1, 3

    LL   = L + J - 1

    LLM1 = LL - 1
    LLP1 = LL + 1
    LLP2 = LL + 2

    LZZ = MOD( (MOD(L+3,4) + J), 4 ) + 1

    DO  I = 3, MX2T

! coarse graining
      IO  = IND (I, LL)
      OT  = OTA (IO)
! u_x (I,LL) == u_x (I+1/2,LL+1/2)
      ET1 = ( A*( UT (I+2,LL  ) - UT (I-1,LL  ) )                      &
            + B*( UT (I+1,LL  ) - UT (I  ,LL  ) ) )
! w_z (I,LL) == w_z (I+1/2,LL+1/2)
      ET2 = ( A*( WT (I  ,LLP2) - WT (I  ,LLM1) )                      &
            + B*( WT (I  ,LLP1) - WT (I  ,LL  ) ) )
! xi^zz (I,LL) == xi^zz (I+1/2,LL+1/2)
      XT0 = XZZT (I,LL)
      XT  = OT*XT0 + CYLAM (IO) * ET1 + CYL2M (IO) * ET2
      XZZT   (I,LL ) = XT
! tau_zz (I,LZZ) == tau_zz (I+1/2,LL+1/2)
      TZZMWT (I,LZZ) = COLAM * ET1 + COL2M * ET2                       &
                      - ( XT + XT0 )*0.5_WP

      IF ( KEY_1D ) THEN
        IO     = IND (I+1,LL)
        OT     = OTA (IO)
        XT0    = XXX (I,LL)
        XT     = OT*XT0 + CYLAM (IO) * ET1 + CYL2M (IO) * ET2
        XXX   (I,LL  ) = XT

        TZZMWT (I,LZZ) = TZZMWT (I,LZZ)                                &
                       - ( XT + XT0 ) * 0.5_WP
      END IF

    END DO

  END DO

!_______________________________________________________________________

IF ( .NOT. KEY_1D ) THEN
  L = LOU1

  DO J = 0, 2

    LL   = L + J

    LLM2 = LL - 2
    LLM1 = LL - 1
    LLP1 = LL + 1

    LZX = MOD( (MOD(L+3,4) + J), 4 ) + 1

    DO  I = 3, MX1T

! coarse graining
      IO  = IND (I,LL)
      OT  = OTA (IO)
! u_z (I,LL) + w_x (I,LL) == u_z (I,LL) + w_x (I,LL)
      ET1 = ( A*( UT (I  ,LLP1) - UT (I  ,LLM2)                          &
                + WT (I+1,LL  ) - WT (I-2,LL  ) )                        &
            + B*( UT (I  ,LL  ) - UT (I  ,LLM1)                          &
                + WT (I  ,LL  ) - WT (I-1,LL  ) ) )
! xi^xz (I,LL) == xi^xz (I,LL)
    XT0 = XZXT (I,LL)
    XT  = OT*XT0 + CYMU (IO) * ET1
    XZXT   (I,LL  ) = XT
! tau_xz (I,LZX) == tau_xz (I,LL)
    TZXMWT (I,LZX) = COMU * ET1 - ( XT + XT0 )*0.5_WP

    END DO

  END DO
END IF

END SUBROUTINE  PW_S_INT
