!=======================================================================

SUBROUTINE  REC0 (T, UT, WT, OP_F3, OP_F4)

  USE PRECISION   , ONLY:                                              &
                          WP
  USE AUXIL       , ONLY:                                              &
                          MX1, MX2, MZ2
!                          MZ1, MZ3

  USE CONTROL_DATA, ONLY:                                              &
                          MR, IREC, LREC, KEY_SH, KEY_1D
  USE WAVEFIELD   , ONLY:                                              &
                          SEIS
  IMPLICIT NONE

  INTEGER     ,                  INTENT(IN) :: OP_F3, OP_F4
  REAL    (WP),                  INTENT(IN) :: T
  REAL    (WP), DIMENSION(:,0:), INTENT(IN) :: UT, WT


  INTEGER :: I, J, K, LM1, LM2, L, LP1, LP2, LP3

  REAL(WP), DIMENSION (1:4) :: APROX_Z

!______________________________________________ INTERPOLATATION OF U ___
IF ( .NOT.(KEY_SH) .AND. .NOT.(KEY_1D) ) THEN
  APROX_Z = 0._WP

  DO J = 1, MR


    !------------------------------------------------------ free surface
    IF      ( LREC(J) == 0   ) THEN

      !------------------------- 4th order
      IF      ( ( IREC(J) >= 3 ) .AND. ( IREC(J) <= MX2 ) ) THEN

        L   = 0
        LP1 = L + 1
        LP2 = L + 2
        LP3 = L + 3

        DO K = 4, 1, -1

          I = IREC(J) - K + 3

          APROX_Z (5-K) =                                              &
              35._WP/16._WP * UT (I,L  ) - 35._WP/16._WP * UT (I,LP1)  &
            + 21._WP/16._WP * UT (I,LP2) -  5._WP/16._WP * UT (I,LP3)

        END DO

        SEIS (J) = - 1._WP/6._WP * ( APROX_Z (1) + APROX_Z (4) )       &
                   + 2._WP/3._WP * ( APROX_Z (2) + APROX_Z (3) )

      !------------------------- 2nd order
      ELSE IF ( ( IREC(J) == 2 ) .OR. ( IREC(J) == MX1 ) ) THEN

        L   = 0
        LP1 = L + 1

        DO K = 2, 1, -1

          I = IREC(J) - K + 2

          APROX_Z (3-K) =                                              &
              3._WP/2._WP * UT (I,L  ) - 1._WP/2._WP * UT (I,LP1)

        END DO

        SEIS (J) = 1._WP/2._WP * ( APROX_Z (1) + APROX_Z (2) )

      END IF


    !------------------------------------------------------- LREC(J) = 1
    ELSE IF ( LREC(J) == 1   ) THEN

      !------------------------- 4th order
      IF      ( ( IREC(J) >= 3 ) .AND. ( IREC(J) <= MX2 ) ) THEN

        L   = 1
        LM1 = L - 1
        LP1 = L + 1
        LP2 = L + 2

        DO K = 4, 1, -1

          I = IREC(J) - K + 3

          APROX_Z (5-K) =                                              &
               5._WP/16._WP * UT (I,LM1) + 15._WP/16._WP * UT (I,L  )  &
            -  5._WP/16._WP * UT (I,LP1) +  1._WP/16._WP * UT (I,LP2)

        END DO

        SEIS (J) = - 1._WP/6._WP * ( APROX_Z (1) + APROX_Z (4) )       &
                   + 2._WP/3._WP * ( APROX_Z (2) + APROX_Z (3) )

      !------------------------- 2nd order
      ELSE IF ( ( IREC(J) == 2 ) .OR. ( IREC(J) == MX1 ) ) THEN

        L   = 1
        LM1 = L - 1

        DO K = 2, 1, -1

          I = IREC(J) - K + 2

          APROX_Z (3-K) =                                              &
              1._WP/2._WP * ( UT (I,LM1) + UT (I,L  ) )

        END DO

        SEIS (J) = 1._WP/2._WP * ( APROX_Z (1) + APROX_Z (2) )

      END IF

    !---------------------------------------------------------- interior
    ELSE IF ( LREC(J) <= MZ2 ) THEN

      !------------------------- 4th order
      IF      ( ( IREC(J) >= 3 ) .AND. ( IREC(J) <= MX2 ) ) THEN

        L   = LREC(J)
        LM2 = L - 2
        LM1 = L - 1
        LP1 = L + 1

        DO K = 4, 1, -1

          I = IREC(J) - K + 3

          APROX_Z (5-K) =                                              &
              - 1._WP/6._WP * ( UT (I,LM2) + UT (I,LP1) )              &
              + 2._WP/3._WP * ( UT (I,LM1) + UT (I,L  ) )

        END DO

        SEIS (J) = - 1._WP/6._WP * ( APROX_Z (1) + APROX_Z (4) )       &
                   + 2._WP/3._WP * ( APROX_Z (2) + APROX_Z (3) )

      !------------------------- 2nd order
      ELSE IF ( ( IREC(J) == 2 ) .OR. ( IREC(J) == MX1 ) ) THEN

        L   = LREC(J)
        LM1 = L - 1

        DO K = 2, 1, -1

          I = IREC(J) - K + 2

          APROX_Z (3-K) =                                              &
              1._WP/2._WP * ( UT (I,LM1) + UT (I,L  ) )

        END DO

        SEIS (J) = 1._WP/2._WP * ( APROX_Z (1) + APROX_Z (2) )

      END IF

    END IF


    IF ( ABS(SEIS (J)) .LE. .1E-30 ) SEIS (J) = 0.

  END DO

  WRITE (OP_F3,'(255(X,E12.6))') T, ( SEIS (J), J = 1, MR )
END IF
!_______________________________________________________ W COMPONENT ___

  DO  J = 1, MR

    SEIS (J) = WT ( IREC(J), LREC(J) )

    IF ( ABS(SEIS (J)) .LE. .1E-30 ) SEIS (J) = 0.

  END DO

  WRITE (OP_F4,'(255(X,E12.6))') T, ( SEIS (J), J = 1, MR )


END SUBROUTINE  REC0
