!=======================================================================

SUBROUTINE  TIME_LOOP

  USE PRECISION   , ONLY:                                              &
                          WP
  USE GRID_MEDIUM , ONLY:                                              &
                          H
  USE CONTROL_DATA, ONLY:                                              &
                          MT1, MT2, DT,                                &
                          COVP, COVS,                                  &
                          KLCH, KRCH, KEY_CONT, KEY_1D, KEY_SH
  USE SOURCE      , ONLY:                                              &
                          INC, STH, NT
  USE T_LOOP      , ONLY:                                              &
                          ITIST
  USE NONREF_BOUND, ONLY:                                              &
                          LLBT, LRBT
  USE T_INTERFACES, ONLY:                                              &
                          VEL_ON_SOURCE, VEL_OFF_SOURCE, DISP_UPDATE,  &
                          RECORD_ALL, CLOSE_FILES

  IMPLICIT NONE

  INTEGER :: OFFS
  LOGICAL :: EX

!____________________________________________________ TYPE OF SOURCE ___

  IF ( KEY_1D ) THEN
    WRITE (11,*) ' 1D COMPUTATION '
  END IF

  IF      ( INC .EQ. 0 ) THEN

    IF ( KEY_SH ) THEN
      WRITE (11,*) ' PROPAGATION OF S WAVE '
    ELSE
      WRITE (11,*) ' PROPAGATION OF P WAVE '
    END IF
    STH  = H/COVP
    OFFS = INT ( 5._WP * STH/DT + 1._WP )

  ELSE IF ( INC .EQ. 1 ) THEN

    WRITE (11,*) ' PROPAGATION OF S WAVE '
    STH  = H/COVS
    OFFS = INT ( 5._WP * STH/DT + 1._WP )

  ELSE IF ( INC .EQ. 2 ) THEN

    WRITE (11,*) ' VERTICAL BODY FORCE '
    OFFS = 0

  ELSE IF ( INC .EQ. 3 ) THEN

    WRITE (11,*) ' DOUBLE-COUPLE SOURCE '
    OFFS = 0

  ELSE IF ( INC .EQ. 4 ) THEN

    WRITE (11,*) ' EXPLOSIVE SOURCE '
    OFFS = 0

  ELSE

    WRITE (11,*) ' NONE OF P, OR SV WAVE OR VERTICAL BODY FORCE OR'
    WRITE (11,*) ' DOUBLE-COUPLE SOURCE IS REQUIRED '
    STOP

  END IF

!__________________________________________________________ ON SOURCE __

  DO  ITIST = MT1, MIN(NT+OFFS, MT2)

    PRINT *, 'itist=', ITIST
    INQUIRE ( FILE = 'STOP', EXIST = EX )
    IF ( EX ) THEN
      OPEN  ( 40, FILE='STOP', STATUS='UNKNOWN')
      WRITE ( 40,* ) ITIST
      CLOSE ( 40 )
      IF ( .NOT. (KEY_CONT) ) THEN
        OPEN (50, FILE = 'STOP.CO', FORM='UNFORMATTED', STATUS='NEW' )
      END IF
      CALL RECORD_ALL
      CALL CLOSE_FILES
      STOP
    END IF

    LLBT = ( KLCH < ITIST)
    LRBT = ( KRCH < ITIST)

    CALL  VEL_ON_SOURCE (ITIST)
    CALL  DISP_UPDATE   (ITIST)

  END DO
!_________________________________________________________ OFF SOURCE __

  DO  ITIST = MAX(NT+OFFS+1,MT1), MT2

    PRINT *, 'itist=', ITIST
    INQUIRE ( FILE = 'STOP', EXIST = EX )
    IF ( EX ) THEN
      OPEN  ( 40, FILE='STOP', STATUS='UNKNOWN')
      WRITE ( 40,* ) ITIST
      CLOSE ( 40 )
      IF ( .NOT. (KEY_CONT) ) THEN
        OPEN (50, FILE = 'STOP.CO', FORM='UNFORMATTED', STATUS='NEW' )
      END IF
      CALL RECORD_ALL
      CALL CLOSE_FILES
      STOP
    END IF

    LLBT = ( KLCH < ITIST)
    LRBT = ( KRCH < ITIST)

    CALL  VEL_OFF_SOURCE (ITIST)
    CALL  DISP_UPDATE    (ITIST)

  END DO
!_______________________________________________ RECORD CONTINUE FILE __

  IF  ( KEY_CONT )  CALL  RECORD_ALL

  CALL CLOSE_FILES

END SUBROUTINE  TIME_LOOP
