!=======================================================================

SUBROUTINE  VEL_ON_SOURCE (IT)

  USE PRECISION   , ONLY:                                              &
                          WP
  USE CONTROL_DATA, ONLY:                                              &
                          LHOM, LIN3,                                  &
                          DT, IPAS1, MTS, IPAS2,                       &
                          KEY_TLV, KEY_SNV, KEY_SH, KEY_1D
  USE GRID_MEDIUM , ONLY:                                              &
                          CODEN
  USE AUXIL       , ONLY:                                              &
                          LINH, MZ3, LOU1, LOU2, LOU3,                 &
                                     LIN1, LIN2
  USE WAVEFIELD   , ONLY:                                              &
                          U, W, VU, VW, TU1, TU2, TU3,                 &
                                        TW1, TW2, TW3,                 &
                          XZX, XTZX, XRZX,                             &
                          XZZ, XTZZ, XRZZ, XXX
  USE SOURCE      , ONLY:                                              &
                          INC, STH, ISR, LSR, MXX, MZX, MZZ
  USE T_LOOP      , ONLY:                                              &
                          TR
  USE T_INTERFACES, ONLY:                                              &
                          FS,                                          &
                          HLINE_0, HLINE_1 , HLINE_2  , HLINE_I ,      &
                          HLINE_H, HLINE_PW, HLINE_MZ2, HLINE_MZ1,     &
                          REC, REC0, SNAP
  IMPLICIT NONE

  INTEGER, INTENT(IN):: IT

  INTEGER      :: L
  REAL    (WP) :: FSAUX


  TR = (IT-1) * DT

  IF  ( (INC.EQ.0).OR.(INC.EQ.1) ) THEN

!___________________________________________ TOTAL VELOCITY (0-LOU1) ___

    TU1(:) = U (:, LIN1)
    TW1(:) = W (:, LIN1)

    TU2(:) = U (:, LIN2)
    TW2(:) = W (:, LIN2)

    TU3(:) = U (:, LIN3)
    TW3(:) = W (:, LIN3)

    FSAUX = FS (TR)
    U (:, LIN3) = U (:, LIN3) +    INC * FSAUX
    W (:, LIN3) = W (:, LIN3) + (1-INC)* FSAUX

    FSAUX = FS (TR - STH)
    U (:, LIN2) = U (:, LIN2) +    INC * FSAUX
    W (:, LIN2) = W (:, LIN2) + (1-INC)* FSAUX

    FSAUX = FS (TR - 2._WP*STH)
    U (:, LIN1) = U (:, LIN1) +    INC * FSAUX
    W (:, LIN1) = W (:, LIN1) + (1-INC)* FSAUX

    XZX (:,LIN1:LIN2) = XTZX (:,LIN1:LIN2)
    XZZ (:,LIN1:LIN2) = XTZZ (:,LIN1:LIN2)
    IF ( KEY_1D ) THEN
      XXX (:,LIN1:LIN2) = XTZX (:,LIN1:LIN2)
    END IF

    CALL  HLINE_0

    CALL  HLINE_1

    CALL  HLINE_2

    DO  L = 3, LINH
      CALL HLINE_I (L)
    END DO

    DO  L = LHOM, LOU1
      CALL HLINE_H (L)
    END DO

    U (:, LIN1) = TU1(:)
    W (:, LIN1) = TW1(:)

    U (:, LIN2) = TU2(:)
    W (:, LIN2) = TW2(:)

    U (:, LIN3) = TU3(:)
    W (:, LIN3) = TW3(:)

    XTZX (:,LOU2:LIN2) = XZX (:,LOU2:LIN2)
    XTZZ (:,LOU2:LIN2) = XZZ (:,LOU2:LIN2)
    IF ( KEY_1D ) THEN
      XTZX (:,LOU2:LIN2) = XXX (:,LOU2:LIN2)
    END IF

!_______________________________________ RESIDUAL VELOCITY (LIN1-MZ) ___

    TU1(:) = U (:, LOU1)
    TW1(:) = W (:, LOU1)

    TU2(:) = U (:, LOU2)
    TW2(:) = W (:, LOU2)

    TU3(:) = U (:, LOU3)
    TW3(:) = W (:, LOU3)

    FSAUX = FS (TR - 5._WP*STH)
    U (:, LOU3) = U (:, LOU3) -    INC * FSAUX
    W (:, LOU3) = W (:, LOU3) - (1-INC)* FSAUX

    FSAUX = FS (TR - 4._WP*STH)
    U (:, LOU2) = U (:, LOU2) -    INC * FSAUX
    W (:, LOU2) = W (:, LOU2) - (1-INC)* FSAUX

    FSAUX = FS (TR - 3._WP*STH)
    U (:, LOU1) = U (:, LOU1) -    INC * FSAUX
    W (:, LOU1) = W (:, LOU1) - (1-INC)* FSAUX

    XZX (:,LOU2:LIN2) = XRZX (:,LOU2:LIN2)
    XZZ (:,LOU2:LIN2) = XRZZ (:,LOU2:LIN2)
    IF ( KEY_1D ) THEN
      XXX (:,LOU2:LIN2) = XRZX (:,LOU2:LIN2)
    END IF


    CALL HLINE_PW

    DO  L = LIN1, MZ3
      CALL HLINE_H (L)
    END DO

    CALL HLINE_MZ2

    CALL HLINE_MZ1

    U (:, LOU1) = TU1(:)
    W (:, LOU1) = TW1(:)

    U (:, LOU2) = TU2(:)
    W (:, LOU2) = TW2(:)

    U (:, LOU3) = TU3(:)
    W (:, LOU3) = TW3(:)

    XRZX (:,LOU2:LIN2) = XZX (:,LOU2:LIN2)
    XRZZ (:,LOU2:LIN2) = XZZ (:,LOU2:LIN2)
    IF ( KEY_1D ) THEN
      XRZX (:,LOU2:LIN2) = XXX (:,LOU2:LIN2)
    END IF

!_____________________________________________ XZX, XZZ  (LOU2-LOU1) ___

    XZX (:,LOU2:LOU1) = XTZX (:,LOU2:LOU1)
    XZZ (:,LOU2:LOU1) = XTZZ (:,LOU2:LOU1)
    IF ( KEY_1D ) THEN
      XXX (:,LOU2:LOU1) = XTZX (:,LOU2:LOU1)
    END IF

!_______________________________________________ VERTICAL BODY FORCE ___

  ELSE IF ( INC .EQ. 2 ) THEN

    CALL  HLINE_0

    CALL  HLINE_1

    CALL  HLINE_2

    DO  L = 3, LINH
      CALL HLINE_I (L)
    END DO

    DO  L = LHOM, MZ3
      CALL HLINE_H (L)
    END DO

    CALL HLINE_MZ2

    CALL HLINE_MZ1

    VW (ISR,LSR) = VW (ISR,LSR) + DT*FS(TR)/CODEN

!____________________________________________ HORIZONTAL BODY FORCE ___

  ELSE IF ( INC .EQ. 3 ) THEN

    CALL  HLINE_0

    CALL  HLINE_1

    CALL  HLINE_2

    DO  L = 3, LINH
      CALL HLINE_I (L)
    END DO

    DO  L = LHOM, MZ3
      CALL HLINE_H (L)
    END DO

    CALL HLINE_MZ2

    CALL HLINE_MZ1

    VU (ISR,LSR) = VU (ISR,LSR) + DT*FS(TR)/CODEN

!______________________________________________ DOUBLE-COUPLE SOURCE ___

  ELSE IF ( INC .EQ. 4 ) THEN

    CALL  HLINE_0

    CALL  HLINE_1

    CALL  HLINE_2

    DO  L = 3, LINH
      CALL HLINE_I (L)
    END DO

    DO  L = LHOM, MZ3
      CALL HLINE_H (L)
    END DO

    CALL HLINE_MZ2

    CALL HLINE_MZ1

!________________________________________________________________ Fx ___

    VU ( ISR+1, LSR   ) = VU ( ISR+1, LSR   ) + DT*MXX*FS(TR)/CODEN
    VU ( ISR+1, LSR+1 ) = VU ( ISR+1, LSR+1 ) + DT*MZX*FS(TR)/CODEN
    VU ( ISR  , LSR+1 ) = VU ( ISR  , LSR+1 ) + DT*MZX*FS(TR)/CODEN

    VU ( ISR  , LSR   ) = VU ( ISR  , LSR   ) - DT*MXX*FS(TR)/CODEN
    VU ( ISR+1, LSR-1 ) = VU ( ISR+1, LSR-1 ) - DT*MZX*FS(TR)/CODEN
    VU ( ISR  , LSR-1 ) = VU ( ISR  , LSR-1 ) - DT*MZX*FS(TR)/CODEN

!________________________________________________________________ Fz ___

    VW ( ISR  , LSR+1 ) = VW ( ISR  , LSR+1 ) + DT*MZZ*FS(TR)/CODEN
    VW ( ISR+1, LSR+1 ) = VW ( ISR+1, LSR+1 ) + DT*MZX*FS(TR)/CODEN
    VW ( ISR+1, LSR   ) = VW ( ISR+1, LSR   ) + DT*MZX*FS(TR)/CODEN

    VW ( ISR  , LSR   ) = VW ( ISR  , LSR   ) - DT*MZZ*FS(TR)/CODEN
    VW ( ISR-1, LSR+1 ) = VW ( ISR-1, LSR+1 ) - DT*MZX*FS(TR)/CODEN
    VW ( ISR-1, LSR   ) = VW ( ISR-1, LSR   ) - DT*MZX*FS(TR)/CODEN

!__________________________________________________ EXPLOSIVE SOURCE ___

  ELSE IF ( INC .EQ. 5 ) THEN

    CALL  HLINE_0

    CALL  HLINE_1

    CALL  HLINE_2

    DO  L = 3, LINH
      CALL HLINE_I (L)
    END DO

    DO  L = LHOM, MZ3
      CALL HLINE_H (L)
    END DO

    CALL HLINE_MZ2

    CALL HLINE_MZ1

!________________________________________________________________ Fx ___

!q    VU ( ISR+1, LSR   ) = VU ( ISR+1, LSR   ) + DT*FS(TR)/CODEN
!q    VU ( ISR  , LSR   ) = VU ( ISR  , LSR   ) - DT*FS(TR)/CODEN
    VU ( ISR+1, LSR   ) = VU ( ISR+1, LSR   ) + DT*MXX*FS(TR)/CODEN
    VU ( ISR  , LSR   ) = VU ( ISR  , LSR   ) - DT*MXX*FS(TR)/CODEN

!________________________________________________________________ Fz ___

!q    VW ( ISR  , LSR+1 ) = VW ( ISR  , LSR+1 ) + DT*FS(TR)/CODEN
!q    VW ( ISR  , LSR   ) = VW ( ISR  , LSR   ) - DT*FS(TR)/CODEN
    VW ( ISR  , LSR+1 ) = VW ( ISR  , LSR+1 ) + DT*MZZ*FS(TR)/CODEN
    VW ( ISR  , LSR   ) = VW ( ISR  , LSR   ) - DT*MZZ*FS(TR)/CODEN

 END IF

  IF ( KEY_SH .OR. KEY_1D ) THEN
    VU = 0.
  END IF

!___________________________________________ RECORD FILES - VELOCITY ___

  IF  ( KEY_TLV ) THEN
    IF  ( MOD(IT,IPAS1) .EQ. 0 ) THEN
      CALL  REC  ( T = TR + 0.5_WP*DT, UT    = VU, WT    = VW,         &
                                       OP_F1 = 20, OP_F2 = 21 )
      CALL  REC0 ( T = TR + 0.5_WP*DT, UT    = VU, WT    = VW,         &
                                       OP_F3 = 22, OP_F4 = 23 )
    END IF
  END IF

  IF  ( KEY_SNV ) THEN
    IF  ( IT .LE. MTS ) THEN
      IF  ( MOD(IT,IPAS2) .EQ. 0 ) THEN
        CALL SNAP ( NUMSNAP = IT, UT = VU , WT = VW,                   &
                    SN_BASE_1 = 'SNAP_VU.', SN_BASE_2 = 'SNAP_VW.' )
      END IF
    END IF
  END IF

END SUBROUTINE  VEL_ON_SOURCE
