!=======================================================================
! module contains functions for P and S-wave velocities, density and
! quality factors in the heterogeneous part of the model
!=======================================================================

MODULE MOD_FUNC

  USE MOD_PRECISION, ONLY : WP

  IMPLICIT NONE

!-----------------------------------------------------------------------

  CONTAINS


    !----------------------------------------------------------- Density
    FUNCTION FUNC_RHO(X,Z,LAYER)

      INTEGER , INTENT(IN) :: LAYER
      REAL(WP), INTENT(IN) :: X,Z
      REAL(WP)             :: FUNC_RHO

      IF      ( LAYER == 1 ) THEN             ! first  layer (sediments)
        FUNC_RHO = 2200._WP
      ELSE                                    ! bedrock
        FUNC_RHO = 2600._WP
      END IF

    END FUNCTION FUNC_RHO

    !--------------------------------------------------- P-wave velocity
    FUNCTION FUNC_VP(X,Z,LAYER)

      INTEGER , INTENT(IN) :: LAYER
      REAL(WP), INTENT(IN) :: X,Z
      REAL(WP)             :: FUNC_VP

      IF      ( LAYER == 1 ) THEN             ! first layer (sediments)
        FUNC_VP = 2200._WP
      ELSE                                    ! bedrock
        FUNC_VP = 5000._WP
      END IF

    END FUNCTION FUNC_VP

    !--------------------------------------------------- S-wave velocity
    FUNCTION FUNC_VS(X,Z,LAYER)

      INTEGER , INTENT(IN) :: LAYER
      REAL(WP), INTENT(IN) :: X,Z
      REAL(WP)             :: FUNC_VS

      IF      ( LAYER == 1 ) THEN             ! first layer (sediments)
        FUNC_VS = 590._WP
      ELSE                                    ! bedrock
        FUNC_VS = 2600._WP
      END IF

    END FUNCTION FUNC_VS

    !--------------------------------------------- P-wave quality factor
    FUNCTION FUNC_QP(X,Z,LAYER)

      INTEGER , INTENT(IN) :: LAYER
      REAL(WP), INTENT(IN) :: X,Z
      REAL(WP)             :: FUNC_QP

      IF      ( LAYER == 1 ) THEN             ! first layer (sediments)
        FUNC_QP = 110._WP
      ELSE                                    ! bedrock
        FUNC_QP = 400._WP
      END IF

    END FUNCTION FUNC_QP

    !--------------------------------------------- S-wave quality factor
    FUNCTION FUNC_QS(X,Z,LAYER)

      INTEGER , INTENT(IN) :: LAYER
      REAL(WP), INTENT(IN) :: X,Z
      REAL(WP)             :: FUNC_QS

      IF      ( LAYER == 1 ) THEN             ! first layer (sediments)
        FUNC_QS = 55._WP
      ELSE                                    ! bedrock
        FUNC_QS = 200._WP
      END IF

    END FUNCTION FUNC_QS


END MODULE
