!=======================================================================
! Subroutinw assigns to FD cells with different material parameters
! the different indices
!=======================================================================

SUBROUTINE PARAMETERS_INDICES

  USE MOD_PRECISION , ONLY : WP
  USE MOD_ATTEN     , ONLY : N_FREQ, YMUZX, YMU, YL2M, INDEX_Y
  USE MOD_CONST     , ONLY : NE, NY
  USE MOD_MODEL     , ONLY : MX, LHOM, KEY_Q
  USE MOD_PARAMETERS, ONLY : N_CELLS, RHOU, RHOW, MUZX, MU, L2M, JM, INDEX

!-----------------------------------------------------------------------

  IMPLICIT NONE

  INTEGER :: I, L, J, N, LAST_INDEX, TMP
  LOGICAL :: TEST_NEXT_CELLS, TEST_Q

  REAL(WP), ALLOCATABLE, DIMENSION(:)   :: CELL
  REAL(WP), ALLOCATABLE, DIMENSION(:,:) :: CELL_Y

!-----------------------------------------------------------------------

  ALLOCATE ( CELL(1:NE) )
  ALLOCATE ( INDEX(1:MX*(LHOM+3),1:NE) )
  ALLOCATE ( INDEX_Y(1:MX*(LHOM+3),1:NY,1:N_FREQ) )
  INDEX   = 0._WP
  INDEX_Y = 0._WP

  IF (KEY_Q) THEN
    ALLOCATE ( CELL_Y(1:NY,1:N_FREQ) )
  END IF

  LAST_INDEX = 0

  DO L = 0, LHOM+2
    DO I = 1, MX
    
      CELL(1) = RHOU(I,L)
      CELL(2) = RHOW(I,L)
      CELL(3) = MUZX(I,L)
      CELL(4) = MU  (I,L)
      CELL(5) = L2M (I,L)
      IF (KEY_Q) THEN
        CELL_Y(1,:) = YMUZX(I,L,:)
        CELL_Y(2,:) = YMU  (I,L,:)
        CELL_Y(3,:) = YL2M (I,L,:)
      END IF

      ! the first FD cell
      IF ( LAST_INDEX == 0 ) THEN

        LAST_INDEX = 1
        INDEX(1,:) = CELL(:)
        IF (KEY_Q) INDEX_Y(1,:,:) = CELL_Y(:,:)
        JM(I,L) = LAST_INDEX

      ! all added FD cell
      ELSE


        TMP = 1

        DO N = LAST_INDEX, 1, -1

          TEST_NEXT_CELLS = .FALSE.
          TEST_Q = .TRUE.

          ! testing the   ELASTIC parameters of the new cell with the all added cell
          DO J = 1, NE
            IF ( INDEX(N,J) /= CELL(J) ) THEN   ! new cell and N-th cell are different
              TEST_Q = .FALSE.
              TEST_NEXT_CELLS = .TRUE.
              TMP = TMP + 1
              EXIT
            END IF
          END DO
          ! testing the ANELASTIC parameters of the new cell with the all added cell
          ! (testing only for the FIRST relaxation frequency)
          IF ( TEST_Q .AND. KEY_Q ) THEN
            DO J = 1, NY
              IF ( INDEX_Y(N,J,1) /= CELL_Y(J,1) ) THEN   ! new cell and N-th cell are different
                TEST_NEXT_CELLS = .TRUE.
                TMP = TMP + 1
                EXIT
               END IF
            END DO
          END IF
          
          ! there was found the cell with the same material parameters as new cell
          IF ( .NOT. TEST_NEXT_CELLS ) THEN
            JM(I,L) = N
            EXIT
          END IF

          ! material par. of new cell do not fit with any ones of already added cells;
          ! a new material type was found
          IF ( TMP > LAST_INDEX ) THEN
            LAST_INDEX = LAST_INDEX + 1
            INDEX(LAST_INDEX,:) = CELL(:)
            IF (KEY_Q) INDEX_Y(LAST_INDEX,:,:) = CELL_Y(:,:)
            JM(I,L) = LAST_INDEX
            EXIT
          END IF


        END DO


      END IF

    END DO
  END DO

  N_CELLS = LAST_INDEX

  WRITE(*,*) 'Number of different cells of parameters: ', N_CELLS
  WRITE(99,*) 'Number of different cells of parameters: ', N_CELLS

  DEALLOCATE ( CELL, RHOU, RHOW, MUZX, MU, L2M )
  IF (KEY_Q) DEALLOCATE ( CELL_Y, YMUZX, YMU, YL2M )


END SUBROUTINE PARAMETERS_INDICES
