!=======================================================================
! Preparation of some expresions used for calculation of anelastic coefficients
!=======================================================================

SUBROUTINE PREPARE_ATTEN

  USE MOD_PRECISION , ONLY : WP
  USE MOD_ATTEN     , ONLY : N_FREQ, N2_FREQ, FRJMAX, FRANGE, FREF,    &
                             QP_HOM, QS_HOM, YMU_HOM, YL2M_HOM,        &
                             FRJ, FRK, PLO, THE1, THE2, FJK, FJJ, CF
  USE MOD_CONST     , ONLY : PI
  USE MOD_PARAMETERS, ONLY : MU_HOM, L2M_HOM

!-----------------------------------------------------------------------

  IMPLICIT NONE

  INTEGER  :: I, J, K
  REAL(WP) :: TMP, PF, PFP, THE1S, THE2S, THE1P, THE2P, R

  REAL(WP), DIMENSION( N_FREQ) :: Y
  REAL(WP), DIMENSION(N2_FREQ) :: Q, HEQSW, HEQPW

  COMPLEX(WP), DIMENSION(N2_FREQ) :: SUM1, SUM2

!-----------------------------------------------------------------------

  INTERFACE
    SUBROUTINE     QYLSM(Q,Y)
      USE MOD_PRECISION , ONLY : WP
      USE MOD_ATTEN     , ONLY : N_FREQ, N2_FREQ, FJK, FJJ
      IMPLICIT NONE
      REAL(WP), DIMENSION (N2_FREQ), INTENT( IN) :: Q
      REAL(WP), DIMENSION ( N_FREQ), INTENT(OUT) :: Y
    END SUBROUTINE QYLSM
  END INTERFACE

!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! discretization of frequency intervals
!-----------------------------------------------------------------------

  FREF = 2.*PI * FREF

  DO I = 1, N_FREQ
!    FRJ(I) = FRJMAX / ( 10**( FRANGE * (2.*REAL(I)-1.) / (2*N_FREQ) ) )
    FRJ(I) = FRJMAX / ( 10**( FRANGE * (N_FREQ-REAL(I)) / (N_FREQ-1) ) )
  END DO
  FRJ(:) = 2._WP * PI * FRJ(:)

  DO I = 1, N2_FREQ
!    FRK(I) = FRJMAX / ( 10**( FRANGE *     REAL(I)     / (2*N_FREQ) ) )
    FRK(I) = FRJMAX / ( 10**( FRANGE * ((2*N_FREQ-1)-REAL(I)) / (2*N_FREQ-2) ) )
  END DO
  FRK (:) = 2._WP * PI * FRK(:)

!-----------------------------------------------------------------------
! implementation of Q(w) law
!-----------------------------------------------------------------------

  DO I = 1, N2_FREQ
    PLO(I)= 0.                        !------- constant  Q(w) law
!    PLO(I)= LOG (FRK(I)/FREF) / PI   !------- Futterman Q(w) law
  END DO

!-----------------------------------------------------------------------
! calculation of some parameters
!-----------------------------------------------------------------------

  DO J = 1, N_FREQ

    PF      = FREF / FRJ(J)
    PFP     = 1._WP + PF*PF
    THE1(J) =  1 / PFP
    THE2(J) = PF / PFP

    DO K = 1, N2_FREQ
      FJK(J,K) = FRJ(J)*FRK(K) / ( FRJ(J)*FRJ(J) + FRK(K)*FRK(K) )
      FJJ(J,K) = FRJ(J)*FRJ(J) / ( FRJ(J)*FRJ(J) + FRK(K)*FRK(K) )
      CF (J,K) = FRJ(J) / ( FRJ(J) + CMPLX(0.,FRK(K)) )
    END DO

  END DO

!-----------------------------------------------------------------------
! calculation of anelastic coefficients (YMU_HOM, YL2M_HOM)
! and unrelaxed moduli (MU_HOM, L2M_HOM) for homogeneous halfspace
!-----------------------------------------------------------------------

  DO I = 1, N2_FREQ
    HEQSW(I) = QS_HOM - PLO(I)
    HEQPW(I) = QP_HOM - PLO(I)
  END DO

  ! calculation of anelastic coefficient YMU_HOM and YL2M_HOM
  Q(:) = HEQSW(:)
  CALL QYLSM(Q,Y)
  YMU_HOM(:) = Y(:)

  Q(:) = HEQPW(:)
  CALL QYLSM(Q,Y)
  YL2M_HOM(:) = Y(:)

  ! calculation of unrelaxed moduli MU_HOM and L2M_HOM
  THE1S = 0._WP
  THE2S = 0._WP
  THE1P = 0._WP
  THE2P = 0._WP

  THE1S = 1 - SUM (  YMU_HOM(:)*THE1(:) )
  THE2S =     SUM (  YMU_HOM(:)*THE2(:) )
  THE1P = 1 - SUM ( YL2M_HOM(:)*THE1(:) )
  THE2P =     SUM ( YL2M_HOM(:)*THE2(:) )

        R = SQRT( THE1S*THE1S + THE2S*THE2S )
   MU_HOM =  MU_HOM * (R+THE1S)/(2._WP*R*R)

        R = SQRT( THE1P*THE1P + THE2P*THE2P )
  L2M_HOM = L2M_HOM * (R+THE1P)/(2._WP*R*R)

END SUBROUTINE PREPARE_ATTEN
