!=======================================================================
! Prepares density and elastic moduli at grid line L for integration
!=======================================================================

SUBROUTINE PREPARE_INTEG(LL)

  USE MOD_PRECISION , ONLY : WP
  USE MOD_CONST     , ONLY : C43
  USE MOD_FUNC      , ONLY : FUNC_RHO, FUNC_VP, FUNC_VS
  USE MOD_MODEL     , ONLY : MX, PTS, PTS2, H, H2, HPTS, XBMIN
  USE MOD_PARAMETERS, ONLY : RHOI, MI, KI

!-----------------------------------------------------------------------

  IMPLICIT NONE

  INTEGER, INTENT(IN) :: LL

  INTEGER  :: II, KK, LAYER
  REAL(WP) :: XX, ZZ, RHO, VP, VS, TMP1, TMP2

!-----------------------------------------------------------------------

  INTERFACE
    FUNCTION     GET_LAYER (XX,ZZ)
      USE MOD_PRECISION, ONLY : WP
      USE MOD_MODEL    , ONLY : INT_SED, NI
      IMPLICIT NONE
      INTEGER  :: GET_LAYER
      REAL(WP) :: XX, ZZ
    END FUNCTION GET_LAYER
  END INTERFACE

!----------------------------------------------------------------------

  IF ( LL == 0 ) THEN

    DO KK = 1+PTS, PTS2+PTS

      ZZ = -H2 + (KK-1)*HPTS

      DO II = 1, MX*(PTS2-1)+PTS+1
        XX    = XBMIN-H2 + (II-1)*HPTS
        LAYER = GET_LAYER(XX,ZZ)
        RHO   = FUNC_RHO(XX,ZZ,LAYER)
        VP    = FUNC_VP (XX,ZZ,LAYER)
        VS    = FUNC_VS (XX,ZZ,LAYER)
        TMP1  = RHO*VP*VP
        TMP2  = RHO*VS*VS

        RHOI(II,KK) = RHO                     ! density
        MI  (II,KK) = 1 / TMP2                ! 1/mu
        KI  (II,KK) = 1 / (TMP1-C43*TMP2)     ! 1/kappa
      END DO

    END DO

    RHOI(:,1:PTS) = RHOI(:,PTS2:1+PTS:-1)
    MI  (:,1:PTS) = MI  (:,PTS2:1+PTS:-1)
    KI  (:,1:PTS) = KI  (:,PTS2:1+PTS:-1)

  ELSE

    DO KK = 1, PTS2+PTS

      ZZ = LL*H-H2 + (KK-1)*HPTS

      DO II = 1, MX*(PTS2-1)+PTS+1
        XX    = XBMIN-H2 + (II-1)*HPTS
        LAYER = GET_LAYER(XX,ZZ)
        RHO   = FUNC_RHO(XX,ZZ,LAYER)
        VP    = FUNC_VP (XX,ZZ,LAYER)
        VS    = FUNC_VS (XX,ZZ,LAYER)
        TMP1  = RHO*VP*VP
        TMP2  = RHO*VS*VS

        RHOI(II,KK) = RHO                     ! density
        MI  (II,KK) = 1 / TMP2                ! 1/mu
        KI  (II,KK) = 1 / (TMP1-C43*TMP2)     ! 1/kappa
      END DO

    END DO

  END IF


END SUBROUTINE PREPARE_INTEG
