!=======================================================================
! subroutine determines the density and elastic moduli for all grid points:
!   rho_U(I    ,J+1/2)               muzx(I    ,J    )
!   rho_W(I+1/2,J    )               mu  (I+1/2,J+1/2)
!                                    l2m (I+1/2,J+1/2)
!
! and in case of attenuation the anelastic coefficients for all grid points:
!   Ymuzx(I    ,J    )
!   Ymu  (I+1/2,J+1/2)
!   Yl2m (I+1/2,J+1/2)
!=======================================================================

SUBROUTINE PREPARE_MODEL

  USE MOD_PRECISION , ONLY : WP
  USE MOD_ATTEN     , ONLY : N_FREQ, YMU_HOM, YL2M_HOM,                &
                             YMUZX, YMU, YL2M, YMI, YKI
  USE MOD_CONST     , ONLY : C43
  USE MOD_INTERFACES, ONLY : PREPARE_ATTEN, PREPARE_INTEG,             &
                             PREPARE_INTEG_Y, INT2D, INT2D_Y
  USE MOD_MODEL     , ONLY : NI, PTS, PTS2, H, MX, LHOM, INT_SED, KEY_Q
  USE MOD_PARAMETERS, ONLY : RHO_HOM, MU_HOM, L2M_HOM,                 &
                             RHOU, RHOW, MUZX, MU, L2M, RHOI, MI, KI

!-----------------------------------------------------------------------

  IMPLICIT NONE

  INTEGER  :: I, L, JJ, K
  REAL(WP) :: KAPPA

  REAL(WP), DIMENSION(N_FREQ) :: YKAPPA

!-----------------------------------------------------------------------

  WRITE(*,*) ' Preparing model parameters ...'

  ! calculation of parameters for homogeneous part of the model
  IF (KEY_Q) THEN
    CALL PREPARE_ATTEN
    DO K = 1, N_FREQ
      YMUZX(:,LHOM:LHOM+2,K) =  YMU_HOM(K)         !------------ Ymuzx
      YMU  (:,LHOM:LHOM+2,K) =  YMU_HOM(K)         !------------ Ymu
      YL2M (:,LHOM:LHOM+2,K) = YL2M_HOM(K)         !------------ Yl2m
    END DO
  END IF

  RHOU(:,LHOM:LHOM+2) = RHO_HOM                    !------------ rho_U
  RHOW(:,LHOM:LHOM+2) = RHO_HOM                    !------------ rho_W
  MUZX(:,LHOM:LHOM+2) =  MU_HOM                    !------------ muzx
  MU  (:,LHOM:LHOM+2) =  MU_HOM                    !------------ mu
  L2M (:,LHOM:LHOM+2) = L2M_HOM                    !------------ l2m


  ! calculation of parameters for heterogeneous part of the model
  DO L = 0, LHOM-1
!  DO L = 0, LHOM+2

    CALL PREPARE_INTEG(L)

    IF (KEY_Q) THEN
      CALL PREPARE_INTEG_Y(L)
    END IF

    DO I = 1, MX
    
      JJ = (I-1)*2*PTS

      RHOU(I,L) =   INT2D (RHOI(JJ+1    :JJ+PTS2    ,1+PTS:PTS2+PTS))           ! rho_U
      RHOW(I,L) =   INT2D (RHOI(JJ+1+PTS:JJ+PTS2+PTS,1    :PTS2    ))           ! rho_W
      MUZX(I,L) = 1/INT2D (MI  (JJ+1    :JJ+PTS2    ,1    :PTS2    ))           ! muzx
      MU  (I,L) = 1/INT2D (MI  (JJ+1+PTS:JJ+PTS2+PTS,1+PTS:PTS2+PTS))           ! mu
      KAPPA     = 1/INT2D (KI  (JJ+1+PTS:JJ+PTS2+PTS,1+PTS:PTS2+PTS))           ! kappa
      L2M (I,L) = KAPPA + C43*MU(I,L)                                           ! l2m

      IF (KEY_Q) THEN
        YMUZX(I,L,:) = INT2D_Y (YMI(JJ+1    :JJ+PTS2    ,1    :PTS2    ,:))   ! Ymuzx
        YMU  (I,L,:) = INT2D_Y (YMI(JJ+1+PTS:JJ+PTS2+PTS,1+PTS:PTS2+PTS,:))   ! Ymu
        YKAPPA   (:) = INT2D_Y (YKI(JJ+1+PTS:JJ+PTS2+PTS,1+PTS:PTS2+PTS,:))   ! Ykappa
        YL2M (I,L,:) = ( KAPPA*YKAPPA(:) + C43*MU(I,L)*YMU(I,L,:) ) / L2M(I,L)  ! Yl2m
      END IF

    END DO

  END DO


  DEALLOCATE ( RHOI, MI, KI, INT_SED )
  IF (KEY_Q) DEALLOCATE ( YMI, YKI )

END SUBROUTINE PREPARE_MODEL
