!=======================================================================
! subroutine reads program configuration, set some run-time constants 
! and check if some parameters fulfill required conditions
!=======================================================================

SUBROUTINE SETPARAMS

  USE MOD_PRECISION , ONLY : WP
  USE MOD_ATTEN     , ONLY : QP_HOM, QS_HOM, FRJMAX, FRANGE, FREF
  USE MOD_FILES     , ONLY : JM_FILE_NAME, MO_FILE_NAME, Q_FILE_NAME,  &
                             FILE_SED
  USE MOD_INTERFACES, ONLY : PRINT_INFO
  USE MOD_MODEL     , ONLY : H, NP, NI, MX, LHOM, HPTS, PTS, PTS2,     &
                             XBMIN, XIMIN, XBMAX, XIMAX,               &
                             ZHOM, H2, KEY_Q
  USE MOD_PARAMETERS, ONLY : RHO_HOM, VP_HOM, VS_HOM, MU_HOM, L2M_HOM

!----------------------------------------------------------------------

  IMPLICIT NONE

  INTEGER  :: I
  REAL(WP) :: TMP1, TMP2
  LOGICAL  :: ERROR      

  NAMELIST /CONTROL/ H, NP, NI, PTS, KEY_Q

  NAMELIST /BOUNDS/  XBMIN, XBMAX, ZHOM

  NAMELIST /FILES/   JM_FILE_NAME, MO_FILE_NAME, Q_FILE_NAME, FILE_SED

  NAMELIST /PARAMS/  RHO_HOM, VP_HOM, VS_HOM, QP_HOM, QS_HOM,          &
                     FRJMAX, FRANGE, FREF

!----------------------------------------------------------------------

  OPEN (10, FILE='2DMODEL', STATUS='OLD' )
    READ (10, NML = CONTROL  )
    READ (10, NML = BOUNDS   )
    READ (10, NML = FILES    )
    READ (10, NML = PARAMS   )
  CLOSE(10)

!------------------------------------------- set some run-time constants

                     ! -------------------------------- model parameters
  MX   = (XBMAX - XBMIN) / H + 1
  LHOM = ABS(ZHOM) / H
  H2   = H / 2._WP
  PTS2 = PTS*2 + 1
  HPTS = H / (PTS2-1)

  XIMIN = XBMIN - H
  XIMAX = XBMAX + H

  NI = NI+1

                     ! ---------------------------- materaial parameters
                     !                      for homog. part of the model
   MU_HOM = RHO_HOM * VS_HOM * VS_HOM
  L2M_HOM = RHO_HOM * VP_HOM * VP_HOM


                     ! ----------------------- tests of input parameters
  ERROR = .FALSE.

  TMP1 = (XBMAX - XBMIN) / H
  TMP2 = ABS( ( (TMP1) - ANINT(TMP1) ) / TMP1 )
  IF ( TMP2 > 1.E-4_WP ) THEN
    WRITE(*,*          ) '(XBMAX-XBMIN)/H MUST BY AN INTEGER NUMBER.'
    WRITE(*,'(A,D15.6)') 'ACTUAL VALUE: ', TMP1
    ERROR = .TRUE.
  END IF

  TMP1 = ABS(ZHOM) / H
  TMP2 = ABS( ( (TMP1) - ANINT(TMP1) ) / TMP1 )
  IF ( TMP2 > 1.E-4_WP ) THEN
    WRITE(*,'(A)'      ) 'ABS(ZH)/H MUST BY AN INTEGER NUMBER.'
    WRITE(*,'(A,D15.6)') 'ACTUAL VALUE: ', TMP1
    ERROR = .TRUE.
  END IF

  IF ( (PTS <= 0 ) .OR. ( MOD(PTS,2) /=0 ) ) THEN
    WRITE(*,'(A)'   ) 'PTS MUST BE EVEN AND > 0.'
    WRITE(*,'(A,I5)') 'ACTUAL VALUE: ', PTS
    ERROR = .TRUE.
  END IF


  IF (ERROR) STOP

  !------------------------------------ print info about model on screen
  CALL PRINT_INFO

END SUBROUTINE SETPARAMS
